@@ -39,11 +39,12 @@
 //
 // Implementation of Netscape entry points (NPN_*)
 //
-#include "npapi.h"
-#include "npupp.h"
+#include <npapi.h>
+//#include "npupp.h"
+#include <npfunctions.h>
 
 #ifndef HIBYTE
-#define HIBYTE(x) ((((uint32)(x)) & 0xff00) >> 8)
+#define HIBYTE(x) ((((uint32_t)(x)) & 0xff00) >> 8)
 #endif
 
 #ifndef LOBYTE
@@ -78,7 +79,7 @@ NPError NPN_GetURL(NPP instance, const char *url, const char *target)
     return rv;
 }
 
-NPError NPN_PostURLNotify(NPP instance, const char *url, const char *window, uint32 len,
+NPError NPN_PostURLNotify(NPP instance, const char *url, const char *window, uint32_t len,
                           const char *buf, NPBool file, void *notifyData)
 {
     int navMinorVers = NPNFuncs.version & 0xFF;
@@ -92,8 +93,8 @@ NPError NPN_PostURLNotify(NPP instance, const char *url, const char *window, uin
     return rv;
 }
 
-NPError NPN_PostURL(NPP instance, const char *url, const char *window, uint32 len, const char *buf,
-                    NPBool file)
+NPError NPN_PostURL(NPP instance, const char *url, const char *window, uint32_t len,
+                    const char *buf, NPBool file)
 {
     NPError rv = NPNFuncs.posturl(instance, url, window, len, buf, file);
     return rv;
@@ -119,10 +120,10 @@ NPError NPN_NewStream(NPP instance, NPMIMEType type, const char *target, NPStrea
     return rv;
 }
 
-int32 NPN_Write(NPP instance, NPStream * stream, int32 len, void *buffer)
+int32_t NPN_Write(NPP instance, NPStream * stream, int32_t len, void *buffer)
 {
     int navMinorVersion = NPNFuncs.version & 0xFF;
-    int32 rv = 0;
+    int32_t rv = 0;
 
     if (navMinorVersion >= NPVERS_HAS_STREAMOUTPUT)
         rv = NPNFuncs.write(instance, stream, len, buffer);
@@ -157,7 +158,7 @@ const char *NPN_UserAgent(NPP instance)
     return rv;
 }
 
-void *NPN_MemAlloc(uint32 size)
+void *NPN_MemAlloc(uint32_t size)
 {
     void *rv = NULL;
     rv = NPNFuncs.memalloc(size);
@@ -169,9 +170,9 @@ void NPN_MemFree(void *ptr)
     NPNFuncs.memfree(ptr);
 }
 
-uint32 NPN_MemFlush(uint32 size)
+uint32_t NPN_MemFlush(uint32_t size)
 {
-    uint32 rv = NPNFuncs.memflush(size);
+    uint32_t rv = NPNFuncs.memflush(size);
     return rv;
 }
 
@@ -180,6 +181,7 @@ void NPN_ReloadPlugins(NPBool reloadPages)
     NPNFuncs.reloadplugins(reloadPages);
 }
 
+/*
 JRIEnv *NPN_GetJavaEnv(void)
 {
     JRIEnv *rv = NULL;
@@ -193,6 +195,7 @@ jref NPN_GetJavaPeer(NPP instance)
     rv = NPNFuncs.getJavaPeer(instance);
     return rv;
 }
+*/
 
 NPError NPN_GetValue(NPP instance, NPNVariable variable, void *value)
 {
