@@ -50,6 +50,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
     gchar *path;
     CPlugin *instance;
     ListItem *item = NULL;
+    ListItem *fetch_item = NULL;
     gchar *arg[10];
     gint i;
     GRand *rand;
@@ -120,7 +121,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
 
             if (g_ascii_strcasecmp(dbus_message_get_member(message), "ListDump") == 0) {
 
-                printf("playlist:\n");
+                gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "playlist:\n");
                 list_dump(instance->playlist);
 
                 return DBUS_HANDLER_RESULT_HANDLED;
@@ -129,19 +130,23 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
             if (g_ascii_strcasecmp(dbus_message_get_member(message), "RequestById") == 0) {
                 dbus_error_init(&error);
                 if (dbus_message_get_args(message, &error, DBUS_TYPE_STRING, &s, DBUS_TYPE_INVALID)) {
-                    printf("Got id %s\n", s);
+                    gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Got id %s\n", s);
                     item = list_find_by_id(instance->playlist, (gint) g_strtod(s, NULL));
                     if (item != NULL) {
                         item->play = TRUE;
                         item->cancelled = FALSE;
                         item->retrieved = FALSE;
-                        printf("id %s has url of %s\n", s, item->src);
-                        printf("id %s has newwindow = %i\n", s, item->newwindow);
+                        gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "id %s has url of %s\n", s,
+                               item->src);
+                        gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+                               "id %s has newwindow = %i\n", s, item->newwindow);
                         if (item->newwindow == 0) {
                             send_signal_with_boolean(instance, item, "SetShowControls", TRUE);
                             if (item->streaming) {
                                 send_signal_with_string(instance, item, "Open", item->src);
                             } else {
+                                item->requested = TRUE;
+                                item->queuedtoplay = TRUE;
                                 instance->GetURLNotify(instance->mInstance, item->src, NULL, item);
                             }
                         } else {
@@ -172,12 +177,16 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                             if (g_spawn_async(NULL, arg, NULL,
                                               G_SPAWN_SEARCH_PATH, NULL, NULL, NULL,
                                               &gerror) == FALSE) {
-                                printf("Unable to launch %s: %s\n", app_name, gerror->message);
+                                gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+                                       "Unable to launch %s: %s\n", app_name, gerror->message);
                                 g_error_free(gerror);
                                 gerror = NULL;
                             }
                             g_free(app_name);
-                            printf("requesting %s \n", item->src);
+                            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "requesting %s \n",
+                                   item->src);
+                            item->requested = TRUE;
+                            item->queuedtoplay = TRUE;
                             instance->GetURLNotify(instance->mInstance, item->src, NULL, item);
                         }
                         instance->lastopened->played = TRUE;
@@ -218,9 +227,14 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                     }
                 }
 
-                //printf("next item src = %s\n", item->src);
-                //printf("next item local = %s\n", item->local);
-                //printf("next item streaming = %i\n", item->streaming);
+                if (item != NULL) {
+                    printf("next item src = %s\n", item->src);
+                    printf("next item local = %s\n", item->local);
+                    printf("next item streaming = %i\n", item->streaming);
+                } else {
+                    printf("do not know what the next item is\n");
+                    list_dump(instance->playlist);
+                }
 
                 if (item != NULL) {
                     /*
@@ -249,11 +263,35 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                         item->cancelled = FALSE;
                         if (item->retrieved) {
                             open_location(instance, item, TRUE);
+                            fetch_item =
+                                list_find_next_playable_after_listitem(instance->playlist, item);
+                            if (fetch_item != NULL) {
+                                if (!fetch_item->streaming) {
+                                    gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+                                           "Getting URL '%s'", fetch_item->src);
+                                    fetch_item->requested = TRUE;
+                                    instance->GetURLNotify(instance->mInstance, fetch_item->src,
+                                                           NULL, fetch_item);
+                                }
+                            }
                         } else {
+                            item->requested = TRUE;
+                            item->queuedtoplay = TRUE;
                             instance->GetURLNotify(instance->mInstance, item->src, NULL, item);
                         }
                     } else {
                         open_location(instance, item, FALSE);
+                        fetch_item =
+                            list_find_next_playable_after_listitem(instance->playlist, item);
+                        if (fetch_item != NULL) {
+                            if (!fetch_item->streaming) {
+                                gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Getting URL '%s'",
+                                       fetch_item->src);
+                                fetch_item->requested = TRUE;
+                                instance->GetURLNotify(instance->mInstance, fetch_item->src, NULL,
+                                                       fetch_item);
+                            }
+                        }
                     }
                 }
                 return DBUS_HANDLER_RESULT_HANDLED;
@@ -271,21 +309,25 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                             NPN_GetURL(instance->mInstance, instance->event_mediacomplete, NULL);
                         }
                         postPlayStateChange(instance->mInstance, STATE_MEDIAENDED);
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MouseClicked") == 0) {
                         if (instance->event_mouseclicked != NULL) {
                             NPN_GetURL(instance->mInstance, instance->event_mouseclicked, NULL);
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "EnterWindow") == 0) {
                         if (instance->event_enterwindow != NULL) {
                             NPN_GetURL(instance->mInstance, instance->event_enterwindow, NULL);
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "LeaveWindow") == 0) {
                         if (instance->event_leavewindow != NULL) {
                             NPN_GetURL(instance->mInstance, instance->event_leavewindow, NULL);
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MouseDown") == 0) {
                         if (instance->event_mousedown != NULL) {
@@ -293,6 +335,7 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                             NPN_GetURL(instance->mInstance, tmp, NULL);
                             g_free(tmp);
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MouseUp") == 0) {
                         if (instance->event_mouseup != NULL) {
@@ -300,36 +343,43 @@ static DBusHandlerResult filter_func(DBusConnection * connection,
                             NPN_GetURL(instance->mInstance, tmp, NULL);
                             g_free(tmp);
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "TimeChanged") == 0) {
                         if (instance->post_dom_events && instance->id != NULL) {
                             postDOMEvent(instance->mInstance, instance->id, "qt_timechanged");
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "Ended") == 0) {
                         if (instance->post_dom_events && instance->id != NULL) {
                             postDOMEvent(instance->mInstance, instance->id, "qt_ended");
                         }
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MediaStopped") == 0) {
                         if (instance->post_dom_events && instance->id != NULL) {
                             postDOMEvent(instance->mInstance, instance->id, "qt_ended");
                         }
                         postPlayStateChange(instance->mInstance, STATE_STOPPED);
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MediaPlaying") == 0) {
                         if (instance->post_dom_events && instance->id != NULL) {
                             postDOMEvent(instance->mInstance, instance->id, "qt_play");
                         }
                         postPlayStateChange(instance->mInstance, STATE_PLAYING);
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                     if (g_ascii_strcasecmp(s, "MediaPaused") == 0) {
                         if (instance->post_dom_events && instance->id != NULL) {
                             postDOMEvent(instance->mInstance, instance->id, "qt_pause");
                         }
                         postPlayStateChange(instance->mInstance, STATE_PAUSED);
+                        return DBUS_HANDLER_RESULT_HANDLED;
                     }
                 }
+                return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
             }
         }
     } else {
@@ -352,7 +402,8 @@ DBusConnection *dbus_hookup(CPlugin * instance)
     dbus_bus_add_match(connection, "type='signal',interface='com.gecko.mediaplayer'", NULL);
     dbus_connection_add_filter(connection, filter_func, instance, NULL);
 
-    printf("DBUS connection created\nListening to path %s\n", instance->path);
+    gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+           "DBUS connection created\nListening to path %s\n", instance->path);
 
     return connection;
 }
@@ -396,6 +447,10 @@ void open_location(CPlugin * instance, ListItem * item, gboolean uselocal)
                 file = g_strdup(item->src);
             }
 
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+                   "Opening %s with launch\nitem->hrefid = %i item->src = %s", file, item->hrefid,
+                   item->src);
+
             //printf("launching gnome-mplayer from Open with id = %i\n",instance->controlid);
             app_name = NULL;
             if (instance->player_backend != NULL) {
@@ -458,7 +513,8 @@ void open_location(CPlugin * instance, ListItem * item, gboolean uselocal)
             path = instance->path;
         }
 
-        printf("Sending Open %s to connection %p\nitem->hrefid = %i item->src = %s\n", file,
+        gm_log(instance->debug_level, G_LOG_LEVEL_INFO,
+               "Sending Open %s to connection %p\nitem->hrefid = %i item->src = %s\n", file,
                instance->connection, item->hrefid, item->src);
         if (item->hrefid == 0) {
             if (item->streaming) {
@@ -487,7 +543,8 @@ void open_location(CPlugin * instance, ListItem * item, gboolean uselocal)
         item->opened = TRUE;
         instance->lastopened = item;
     } else {
-        printf("Item already opened before\n");
+        gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Item '%s' already opened before",
+               item->src);
     }
 }
 
@@ -588,9 +645,10 @@ void send_signal_with_string(CPlugin * instance, ListItem * item, const gchar *
     const char *localstr;
     gchar *path;
 
-    printf("Sending %s to connection %p\n", signal, instance->connection);
     if (instance == NULL)
         return;
+//    gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Sending %s to connection %p\n", signal,
+//           instance->connection);
 
     if (instance->console != NULL) {
         path = g_strdup_printf("/console/%s", instance->console);
@@ -620,10 +678,12 @@ void send_signal_with_double(CPlugin * instance, ListItem * item, const gchar *
     const char *localsignal;
     gchar *path;
 
-    //printf("Sending %s to connection %p\n", signal, instance->connection);
     if (instance == NULL)
         return;
 
+//    gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Sending %s to connection %p\n", signal,
+//           instance->connection);
+
     if (instance->console != NULL) {
         path = g_strdup_printf("/console/%s", instance->console);
     } else {
@@ -741,7 +801,7 @@ gboolean request_boolean_value(CPlugin * instance, ListItem * item, const gchar
         replymessage =
             dbus_connection_send_with_reply_and_block(instance->connection, message, -1, &error);
         if (dbus_error_is_set(&error)) {
-            printf("Error message = %s\n", error.message);
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Error message = %s\n", error.message);
         }
         dbus_message_get_args(replymessage, &error, DBUS_TYPE_BOOLEAN, &result, DBUS_TYPE_INVALID);
         dbus_message_unref(message);
@@ -784,7 +844,7 @@ gdouble request_double_value(CPlugin * instance, ListItem * item, const gchar *
         replymessage =
             dbus_connection_send_with_reply_and_block(instance->connection, message, -1, &error);
         if (dbus_error_is_set(&error)) {
-            printf("Error message = %s\n", error.message);
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Error message = %s\n", error.message);
         }
         dbus_message_get_args(replymessage, &error, DBUS_TYPE_DOUBLE, &result, DBUS_TYPE_INVALID);
         dbus_message_unref(message);
@@ -828,7 +888,7 @@ gint request_int_value(CPlugin * instance, ListItem * item, const gchar * member
         replymessage =
             dbus_connection_send_with_reply_and_block(instance->connection, message, -1, &error);
         if (dbus_error_is_set(&error)) {
-            printf("Error message = %s\n", error.message);
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Error message = %s\n", error.message);
         }
         dbus_message_get_args(replymessage, &error, DBUS_TYPE_INT32, &result, DBUS_TYPE_INVALID);
         dbus_message_unref(message);
@@ -871,7 +931,7 @@ gchar *request_string_value(CPlugin * instance, ListItem * item, const gchar * m
         replymessage =
             dbus_connection_send_with_reply_and_block(instance->connection, message, -1, &error);
         if (dbus_error_is_set(&error)) {
-            printf("Error message = %s\n", error.message);
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Error message = %s\n", error.message);
         }
         dbus_message_get_args(replymessage, &error, DBUS_TYPE_STRING, &result, DBUS_TYPE_INVALID);
         dbus_message_unref(message);
@@ -953,7 +1013,7 @@ gint request_bitrate(CPlugin * instance, ListItem * item, gchar * name)
         replymessage =
             dbus_connection_send_with_reply_and_block(instance->connection, message, -1, &error);
         if (dbus_error_is_set(&error)) {
-            printf("Error message = %s\n", error.message);
+            gm_log(instance->debug_level, G_LOG_LEVEL_INFO, "Error message = %s\n", error.message);
         }
         if (replymessage != NULL) {
             dbus_message_get_args(replymessage, &error, DBUS_TYPE_INT32, &result,
